/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Upload = function()
{
	var upload = {};
	
	/**
		Initialize the page.
	*/
	upload.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'select-file')
		{
			// Zip formats
			upload.initZipFormats();
			Events.addEventHandler('tempZipUnpack', 'click', upload.unpackZipOnClick);
			
			// Upload+Close buttons
			Buttons.addClickHandler('btnUpload', upload.doUpload);
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnAbort', upload.abortUpload);

			// File
			Events.addEventHandler('the_file', 'change', upload.fileOnChange);
			
			// Error handling
			Events.addEventHandler(document.body, 'base-notify', upload.onUploadError);
		}
		else if (pageId == 'upload-results')
		{
			var message = Data.get('page-data', 'message');
			var url = '../../common/close_popup.jsp?refresh_opener=1&message='+encodeURIComponent(message);
			window.top.location = url;
		}
	}

	upload.validateFile = function()
	{
		var frm = document.forms['file'];
		if (Strings.trim(frm.the_file.value) == '')
		{
			Forms.showNotification(frm.the_file, 'You must select a file to upload');
			return false;
		}
		return true;
	}
	
	// Submit the form
	upload.doUpload = function()
	{
		var frm = document.forms['file'];
		if (upload.validateFile())
		{
			// Copy form values so they are sent BEFORE the file
			frm.check_md5.value = frm.temp_check_md5 && frm.temp_check_md5.checked ? "1" : "0";
			frm.write_protected.value = frm.temp_write_protected.checked ? "1" : "0";
			frm.replace_existing.value = frm.temp_replace_existing && frm.temp_replace_existing.checked ? "1" : "0";
			frm.compressed.value = frm.temp_compressed && frm.temp_compressed[frm.temp_compressed.selectedIndex].value;

			if (frm.temp_zip_format)
			{
				frm.zip_format.value = frm.temp_zip_format[frm.temp_zip_format.selectedIndex].value;
				frm.zip_unpack.value = frm.temp_zip_unpack.checked ? "1" : "0";
				frm.zip_overwrite.value = frm.temp_zip_overwrite.checked ? "1" : "0";
				frm.zip_keep.value = frm.temp_zip_keep.checked ? "1" : "0";
			}
			frm.characterSet.value = frm.temp_charset[frm.temp_charset.selectedIndex].value;
			frm.subtype_id.value = frm.temp_subtype_id[frm.temp_subtype_id.selectedIndex].value;
			frm.description.value = frm.temp_description.value;
			frm.submit();
			Doc.hide('selectForm');
			Doc.show('progressForm');
			upload.sendProgressUpdateRequest();
		}
	}
	
	upload.fileOnChange = function()
	{
		var frm = document.forms['file'];
		var filename = frm.the_file.value;
		var isZip = false;
		var zipIndex = 0;
		var longestMatch = 0;
		
		for (var i = 0; i < frm.temp_zip_format.length; i++)
		{
			var zf = frm.temp_zip_format[i].zipFormat;
			var matchLength = upload.matchesZipExtension(zf, filename);
			if (matchLength > longestMatch)
			{
				isZip = true;
				zipIndex = i;
				longestMatch = matchLength;
			}
		}
		
		if (frm.temp_zip_format)
		{
			frm['temp_zip_format'].disabled = !isZip;
			frm['temp_zip_format'].selectedIndex = zipIndex;
			frm['temp_zip_unpack'].disabled = !isZip;
			frm['temp_zip_overwrite'].disabled = !isZip || !frm['temp_zip_unpack'].checked;
			frm['temp_zip_keep'].disabled = !isZip || !frm['temp_zip_unpack'].checked;
			frm['temp_compressed'].disabled = isZip;
			if (isZip)
			{
				// Select '-auto-' if present, otherwise 'no'
				Forms.selectListOption(frm['temp_compressed'], '0');
				Forms.selectListOption(frm['temp_compressed'], '');
			}
		}
	}

	upload.initZipFormats = function()
	{
		var frm = document.forms['file'];
		var zipFormats = Data.json('page-data', 'zip-formats');
		
		for (var i = 0; i < zipFormats.length; i++)
		{
			var zf = zipFormats[i];
			var option = new Option(upload.getFullZipFormatName(zf), zf.id);
			frm.temp_zip_format[frm.temp_zip_format.length] = option;
			option.zipFormat = zf;
		}
	}

	upload.getFullZipFormatName = function(zipFormat)
	{
		var s = zipFormat.name + " [";
		for (var i = 0; i < zipFormat.extensions.length; i++)
		{
			if (i > 0) s += ', ';
			s += '*.' + zipFormat.extensions[i];
		}
		s += ']';
		return s;
	}
	
	upload.matchesZipExtension = function(zipFormat, filename)
	{
		var longestMatch = 0;
		for (var i = 0; i < zipFormat.extensions.length; i++)
		{
			var ext = '.' + zipFormat.extensions[i];
			var index = filename.lastIndexOf(ext);
			if (index > 0 && (index + ext.length == filename.length))
			{
				if (ext.length > longestMatch)
				{
					longestMatch = ext.length;
				}
			}
		}
		return longestMatch;
	}
	
	upload.unpackZipOnClick = function()
	{
		var frm = document.forms['file'];
		var checked = frm['temp_zip_unpack'].checked;
		if (!checked) frm['temp_zip_keep'].checked = true;
		frm['temp_zip_overwrite'].disabled = !checked;
		frm['temp_zip_keep'].disabled = !checked;
	}

	upload.sendProgressUpdateRequest = function()
	{
		var request = Ajax.getXmlHttpRequest();
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetProgress&'+Math.random();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, upload.progressUpdateCallback);
		request.send(null);
	}
	
	upload.progressUpdateCallback = function(request)
	{
		var progress = JSON.parse(request.responseText);
		if (progress && progress.status == 'ok' && progress.file)
		{
			var filename = progress.file;
			var transferredBytes = progress.transferredBytes;
			var totalBytes = progress.totalBytes;
			var timeInSeconds = progress.timeInSeconds;
			var message = progress.message;
			upload.displayProgress(filename, transferredBytes, totalBytes, timeInSeconds, message);
		}
		setTimeout(upload.sendProgressUpdateRequest, 1500);
	}

	
	upload.displayProgress = function(fileName, transferredBytes, totalBytes, transferTime, message)
	{
		// File name
		Doc.element('fileName').innerHTML = 'Uploading ' + fileName + '...';
			
		// Percent done
		var percentDone = totalBytes == 0 ? 0 : Math.round(100 * transferredBytes / totalBytes);
		Doc.element('percentText').innerHTML = percentDone+'%';
		var doneElement = Doc.element('percentDone');
		doneElement.style.width = percentDone+'%';
		var remainElement = Doc.element('percentRemain');
		remainElement.style.width = (100-percentDone)+'%';
		if (percentDone == 100) Doc.hide('percentRemain');
		
		// Bytes
		Doc.element('transferredBytes').innerHTML = Numbers.formatBytes(transferredBytes);
		Doc.element('totalBytes').innerHTML = Numbers.formatBytes(totalBytes);

		// Times and rate
		Doc.element('transferTime').innerHTML = Numbers.formatSeconds(transferTime);
		var transferRate = transferTime == 0 ? 0 : transferredBytes / transferTime;
		Doc.element('transferRate').innerHTML = '(' + Numbers.formatBytes(transferRate) + '/s)';
		if (transferRate > 0)
		{
			var remainTime = parseInt((totalBytes-transferredBytes) / transferRate);
			Doc.element('remainTime').innerHTML = Numbers.formatSeconds(remainTime) + ' (estimated)';
		}
		
		// Message
		if (message)
		{
			Doc.element('message').innerHTML = message.replace(/(.*):(.*)/, '<b>$1:</b> $2');
		}
	}


	upload.abortUpload = function()
	{
		if (confirm('Are you sure you want to abort the upload? It cannot be resumed.'))
		{
			/*
			Aborting from server-side is not working
			See ticket: http://base.thep.lu.se/ticket/2018
			Sending "Abort" command to the server and then
			forcefully closing the window and hoping for the 
			best.
			*/
			var request = Ajax.getXmlHttpRequest();
			var url = 'ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=Abort&'+Math.random();
			request.open("GET", url, true);
			request.send(null);
			
			var url = '../../common/close_popup.jsp?refresh_opener=1&message=Upload+aborted';
			window.top.location = url;
		}
	}
	
	upload.onUploadError = function(event)
	{
		if (event.detail.errorWindow == 'upload')
		{
			Doc.show('uploadForm');
			Doc.hide('progressForm');
		}
	}

	return upload;
}();

Doc.onLoad(Upload.initPage);
