<%-- $Id: login.jsp 7410 2017-10-09 11:47:39Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	Login to BASE.
	If successful the request is ultimately forwarded to /user/index.jsp
	If unsuccessful an appropriate error message is displayed.

	@param login The login for the user.
	@param password The password for the user.

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.*"
	import="net.sf.basedb.core.authentication.LoginRequest"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.startpage.StartPageAction"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.Iterator"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.Arrays"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%
	SessionControl sc = Base.getSessionControl(pageContext, true);
	String ID = sc.getId();
	String cmd = request.getParameter("cmd");
	String again = request.getParameter("again");
	boolean useAutoStartPage = false;
	String root = request.getContextPath()+"/";
	String redirect = Values.getStringOrNull(request.getParameter("redirect"));
	String errorTitle = null;
	String errorMessage = null;
	String message = null;
	String login = request.getParameter("login");
		
	if ("Login".equals(cmd) || cmd == null)
	{
		String password = request.getParameter("password");
		String deviceToken = Values.getStringOrNull(request.getParameter("deviceToken"));
		try
		{
			if (sc.isLoggedIn()) sc.logout();
			LoginRequest loginRequest = new LoginRequest(login, password, deviceToken);
			loginRequest.setAttribute("user-agent", request.getHeader("User-Agent"));
			String serverUrl = request.getRequestURL().toString().replace(request.getRequestURI(), root);
			loginRequest.setAttribute("server-url", serverUrl);
			sc.login(loginRequest);
			useAutoStartPage = Values.getBoolean(request.getParameter("useAutoStartPage"));
		}
		catch (DeviceNotVerifiedException ex)
		{
			message = "Device not verified";
			redirect = root + "verify_device.jsp?ID=" + sc.getId();
		}
		catch (LoginException ex)
		{
			errorTitle = "Login failed";
			errorMessage = ex.getMessage();
		}
		catch (ItemNotFoundException ex)
		{
			errorTitle = "Login not found";
			errorMessage = ex.getMessage();
		}
		catch (InvalidPasswordException ex)
		{
			errorTitle = "Invalid password";
			errorMessage = ex.getMessage();
		}
		catch (PermissionDeniedException ex)
		{
			errorTitle = "Permission denied";
			errorMessage = ex.getMessage();
		}
	}
	else if ("VerifyDevice".equals(cmd))
	{
		String verificationCode = request.getParameter("verificationCode");
		boolean rememberDevice = Values.getBoolean(request.getParameter("rememberDevice"));
		try
		{
			sc.verifyDevice(verificationCode, rememberDevice);
		}
		catch (LoginException ex)
		{
			errorTitle = "Login failed";
			errorMessage = ex.getMessage();
		}
	}
	else if ("Impersonate".equals(cmd))
	{
		int userId = Values.getInt(request.getParameter("user_id"));
		String comment = request.getParameter("comment");
		SessionControl impersonated = sc.impersonateLogin(userId, comment);
		impersonated.setSessionSetting("impersonate.originalSessionControl", sc);
		DbControl dc = sc.newDbControl();
		try
		{
			impersonated.setSessionSetting("impersonate.originalUser", User.getById(dc, sc.getLoggedInUserId()));
		}
		finally
		{
			if (dc != null) dc.close();
		}
		sc = impersonated;
		if (redirect == null)
		{
			redirect = root + "my_base/index.jsp?ID=" + sc.getId();
		}
		else
		{
			redirect = redirect.replaceFirst("ID=[a-z0-9]*", "ID=" + sc.getId());
		}
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
	
	if (sc.isLoggedIn())
	{
		// Some roles may be inactive by default
		String tmp = Values.getStringOrNull(sc.getUserClientSetting("inactiveRoles"));
		if (tmp != null)
		{
			Set<Integer> defaultInactive = new HashSet<Integer>(Arrays.asList(Values.getInt(tmp.split(":"))));
			for (int roleId : defaultInactive)
			{
				sc.setRoleInactive(roleId, true);
			}
		}
		
		int lastActiveProjectId = Values.getInt(sc.getUserClientSetting("projects.lastactive"));
		String startPageId = sc.getUserClientSetting("start-page-id");
		
		DbControl dc = sc.newDbControl();
		try
		{
			if (lastActiveProjectId != 0)
			{
				Project p = Project.getById(dc, lastActiveProjectId);
				sc.setActiveProject(p);
			}
			
			if (useAutoStartPage && startPageId != null)
			{
				User user = User.getById(dc, sc.getLoggedInUserId());
				JspContext context = ExtensionsControl.createContext(dc, pageContext, null, user);
				ExtensionsInvoker<StartPageAction> invoker = (ExtensionsInvoker<StartPageAction>)ExtensionsControl.useExtensions(context, "net.sf.basedb.clients.web.start-page");
				Iterator<StartPageAction> it = invoker.iterator();
				while (it.hasNext())
				{
					try
					{
						StartPageAction action = it.next();
						if (startPageId.equals(action.getId()))
						{
							String startPage = action.getUrl();
							if (startPage != null)
							{
								redirect = startPage.replace("$SESSION-ID$", sc.getId());
							}
							break;
						}
					}
					catch (RuntimeException ex)
					{}
				}
			}
		}
		catch (Throwable t)
		{}
		finally
		{
			if (dc != null) dc.close();
		}
	
	}

	if ("1".equals(again))
	{
		if (errorMessage != null)
		{
			throw new WebException("popup", errorTitle, errorMessage);
		}
		else
		{
			if (message == null) message = "Login successful";
			if (redirect == null)
			{
				response.sendRedirect(root+"common/close_popup.jsp?message="+HTML.urlEncode(message)+"&refresh_opener=1");
			}
			else
			{
				response.sendRedirect(root+"common/close_popup.jsp?message="+HTML.urlEncode(message)+"&redirect_opener="+HTML.urlEncode(redirect));
			}
		}
	}
	else
	{
		if (errorMessage != null)
		{
			response.sendRedirect(root+"main.jsp?ID="+ID+"&login="+HTML.urlEncode(login)+"&error="+HTML.urlEncode(errorMessage));
		}
		else
		{
			if (redirect == null)
			{
				response.sendRedirect(root+"my_base/index.jsp?refresh_menu=1&ID="+ID);
			}
			else
			{
				response.sendRedirect(redirect);
			}
		}
	}
%>
